#!/bin/sh -e

#
# Copyright (c) 2022 VMware, Inc. All Rights Reserved.
#

set -e -o pipefail

# Init Java options - ORG_JAVA_OPTS will keep the original value when the script was started for the first time
JAVA_OPTS="${ORG_JAVA_OPTS:=${JAVA_OPTS:- }}"
export ORG_JAVA_OPTS

# Allow custom props to be inserted
test -f /custom-conf1.sh && . /custom-conf1.sh

# remove options not supported in OpenJDK 11
JAVA_OPTS=$( <<< "$JAVA_OPTS" sed -e 's/-XX:+UseParNewGC//' )

# General Java options
JAVA_OPTS+=" -showversion"
JAVA_OPTS+=" -XX:+PrintCommandLineFlags"
JAVA_OPTS+=" -XX:+PrintStringTableStatistics"
JAVA_OPTS+=" -XX:ErrorFile=/var/log/error_java_%p.log"
JAVA_OPTS+=" -Xlog:gc*:file=/var/log/gc.log:time,uptime,level,tags:filecount=10,filesize=10m"

# memory options
# use parallel GC if no GC is added in JAVA_OPTS - the default G1 uses too much off-heap memory
if ! grep -E 'UseG1GC|UseConcMarkSweepGC|UseEpsilonGC|UseParallelGC|UseSerialGC|UseZGC' <<< "$JAVA_OPTS" > /dev/null
then
  echo "Setting GC to ParallelGC"
  JAVA_OPTS+=" -XX:+UseParallelGC"
fi

echo "Setting heap to ${JVM_HEAP:=1024m}"
JAVA_OPTS+=" -Xms$JVM_HEAP -Xmx$JVM_HEAP"

echo "Setting metaspace to ${JVM_METASPACE:=256m}"
JAVA_OPTS+=" -XX:MaxMetaspaceSize=$JVM_METASPACE"

HEAP_DUMP_PATH="/var/log/$( date '+%Y_%m_%d_%H_%M' )_heap_dump.hprof"

JAVA_OPTS+=" -Xss512K"
JAVA_OPTS+=" -XX:+HeapDumpOnOutOfMemoryError -XX:HeapDumpPath=$HEAP_DUMP_PATH"
JAVA_OPTS+=" -XX:+ExitOnOutOfMemoryError"
JAVA_OPTS+=" -XX:+UseStringDeduplication"


if [ -n "$DEBUG_PORT" ]; then
  echo "Enabling debugging on port: $DEBUG_PORT"
  JAVA_OPTS+=" -Xdebug -Xrunjdwp:transport=dt_socket,suspend=n,server=y,address=$DEBUG_PORT"
fi

if [ -n "$SERVER_PORT" ]; then
  JAVA_OPTS+=" -Dserver.port=$SERVER_PORT"
fi

if [ -n "$JMX_PORT" ]; then
  echo "Enabling JMX connection on port: $JMX_PORT"
  JAVA_OPTS+=" -Dcom.sun.management.jmxremote"
  JAVA_OPTS+=" -Dcom.sun.management.jmxremote.port=$JMX_PORT"
  JAVA_OPTS+=" -Dcom.sun.management.jmxremote.rmi.port=$JMX_PORT"
  JAVA_OPTS+=" -Djava.rmi.server.hostname=${JMX_HOST:-127.0.0.1}"
  JAVA_OPTS+=" -Dcom.sun.management.jmxremote.ssl=false"
  JAVA_OPTS+=" -Dcom.sun.management.jmxremote.local.only=false"
  JAVA_OPTS+=" -Dcom.sun.management.jmxremote.authenticate=false"
fi

if [ -n "$EXT_JAVA_OPTS" ]; then
  echo "External java options: $EXT_JAVA_OPTS"
  JAVA_OPTS+=" $EXT_JAVA_OPTS"
fi

# Add ingress/load-balancert/proxy certificates in the java keystore if such are mounted,
# configure FIPS properties
#
source /usr/local/bin/setup_security.sh

if [ -n "$JMX_PORT" ]; then
  echo "Enabling JMX connection on port: $JMX_PORT"
  JAVA_OPTS+=" -Dcom.sun.management.jmxremote"
  JAVA_OPTS+=" -Dcom.sun.management.jmxremote.port=$JMX_PORT"
  JAVA_OPTS+=" -Dcom.sun.management.jmxremote.rmi.port=$JMX_PORT"
  JAVA_OPTS+=" -Djava.rmi.server.hostname=${JMX_HOST:-127.0.0.1}"
  JAVA_OPTS+=" -Dcom.sun.management.jmxremote.ssl=false"
  JAVA_OPTS+=" -Dcom.sun.management.jmxremote.local.only=false"
  JAVA_OPTS+=" -Dcom.sun.management.jmxremote.authenticate=false"
fi

if [ -n "$EXT_JAVA_OPTS" ]; then
  echo "External java options: $EXT_JAVA_OPTS"
  JAVA_OPTS+=" $EXT_JAVA_OPTS"
fi

HUB_SM_CERT_PATH=/run/pinniped/certs

if [ -d  $HUB_SM_CERT_PATH ]
then
  read TRUST_STORE_PASSWORD < $JAVA_HOME/lib/security/cacerts.pass
  openssl pkcs12 -export -out /tmp/combined.pfx -inkey /run/pinniped/certs/tls.key -in /run/pinniped/certs/tls.crt -passout pass:
  openssl pkcs12 -in /tmp/combined.pfx -clcerts -nokeys -out /tmp/combined.cer -passin pass:
  keytool -noprompt -keystore "$JAVA_HOME/lib/security/cacerts" -alias pinniped -import -file "/tmp/combined.cer" -storepass $TRUST_STORE_PASSWORD -trustcacerts
  echo "Importing pinniped certs $JAVA_HOME/lib/security/cacerts"
  JAVA_OPTS+=" -Djavax.net.ssl.trustStore=$JAVA_HOME/lib/security/cacerts"
  JAVA_OPTS+=" -Djavax.net.ssl.trustStorePassword=$TRUST_STORE_PASSWORD"
fi


SSL_CERT_PATH=/run/ensemble-endpoint-manager/ssl-certs

if [ -d  $SSL_CERT_PATH ]
then
  SSL_MOUNT_PATH=/run/ensemble-endpoint-manager/kafka/ssl
  SSL_KEY_PASSWORD=$SSL_KEY_PASSWORD
  KEYSTORE_LOCATION="/run/ensemble-endpoint-manager/kafka/ssl/ensemble-endpoint-manager.diploma.keystore.jks"
  TRUSTSTORE_LOCATION="/run/ensemble-endpoint-manager/kafka/ssl/ensemble-endpoint-manager.diploma.truststore.jks"

  echo "-----------------------"
  echo $KEYSTORE_LOCATION
  echo $TRUSTSTORE_LOCATION
  echo "-----------------------"

  # We use a k8s construct called emptyDir that survives container crashes.
  # Therefore we need to manually clean up contents of the emptyDir every time
  # this script runs to prevent errors of the keytool trying to import
  # same entry twice and can also pick up renewed certs.
  rm -rf $TRUSTSTORE_LOCATION
  rm -rf $KEYSTORE_LOCATION
  rm -rf $SSL_MOUNT_PATH/localhost.p12
  rm -rf $SSL_MOUNT_PATH/root-bundle.pem

  cat $SSL_CERT_PATH/root*.pem > $SSL_MOUNT_PATH/root-bundle.pem
  # The PKCS#12 or PFX format is a binary format for storing the server certificate,
  # any intermediate certificates, and the private key into a single encryptable file.
  # Below command creates a .p12 file that contains the pem files containing
  # certificate chains (root, intermediate, and cert) and the corresponding
  # key
  openssl pkcs12 -export -in $SSL_CERT_PATH/chain.pem -inkey $SSL_CERT_PATH/key.pem \
    -out $SSL_MOUNT_PATH/localhost.p12 -name localhost -CAfile $SSL_MOUNT_PATH/root-bundle.pem \
    -passout pass:$SSL_KEY_PASSWORD

  # Below command creates a keystore at runtime.
  keytool -importkeystore -deststorepass $SSL_KEY_PASSWORD -destkeypass \
    $SSL_KEY_PASSWORD -destkeystore $KEYSTORE_LOCATION -srckeystore $SSL_MOUNT_PATH/localhost.p12 \
    -srcstoretype PKCS12 -srcstorepass $SSL_KEY_PASSWORD -alias localhost

  # Below command creates a truststore at runtime
  COUNTER=0
  for filename in `ls $SSL_CERT_PATH/root*.pem`; do
    echo "Importing root cert $filename"
    keytool -noprompt -keystore $TRUSTSTORE_LOCATION -alias diploma-$COUNTER \
      -import -file $filename -storepass $SSL_KEY_PASSWORD -trustcacerts
    COUNTER=$((COUNTER+1))
  done
fi

#https://www.oracle.com/java/technologies/javase/jdk-11-relnote.html
#TLS 1.3 Half-Close Policy
JAVA_OPTS+=" -Djdk.tls.acknowledgeCloseNotify=true"

# To run ignite with Java 17
JAVA_OPTS+=" --add-opens=jdk.management/com.sun.management.internal=ALL-UNNAMED"
JAVA_OPTS+=" --add-opens=java.base/jdk.internal.misc=ALL-UNNAMED"
JAVA_OPTS+=" --add-opens=java.base/sun.nio.ch=ALL-UNNAMED"
JAVA_OPTS+=" --add-opens=java.management/com.sun.jmx.mbeanserver=ALL-UNNAMED"
JAVA_OPTS+=" --add-opens=jdk.internal.jvmstat/sun.jvmstat.monitor=ALL-UNNAMED"
JAVA_OPTS+=" --add-opens=java.base/sun.reflect.generics.reflectiveObjects=ALL-UNNAMED"
JAVA_OPTS+=" --add-opens=java.base/java.io=ALL-UNNAMED"
JAVA_OPTS+=" --add-opens=java.base/java.nio=ALL-UNNAMED"
JAVA_OPTS+=" --add-opens=java.base/java.util=ALL-UNNAMED"
JAVA_OPTS+=" --add-opens=java.base/java.util.concurrent=ALL-UNNAMED"
JAVA_OPTS+=" --add-opens=java.base/java.util.concurrent.locks=ALL-UNNAMED"
JAVA_OPTS+=" --add-opens=java.base/java.lang=ALL-UNNAMED"

echo -e '\n\n==============\n\n\n'
status=0

##### Start the programs

touch /var/log/ensemble-app.log
tail -c 0 -F /var/log/ensemble-app.log &
TAILPID=$!

cd "$ENSEMBLE_HOME"

java \
    $JAVA_OPTS \
    -cp ${BCFIPS_JARS:+"$BCFIPS_JARS"}:. \
    -jar $ENSEMBLE_HOME/ensemble-app.jar --spring.config.additional-location="$APPLICATION_CONFIG_FILE"\
    "$@" 2>&1 &

PID=$!

wait $PID || status=$?
echo "jvm exited with status code $status"
kill $TAILPID || true

exit "$status"
